//Copyright 2016 Ryan Wick

//This file is part of Bandage

//Bandage is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//Bandage is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with Bandage.  If not, see <http://www.gnu.org/licenses/>.


#ifndef BUILDBLASTDATABASEWORKER_H
#define BUILDBLASTDATABASEWORKER_H

#include <QObject>
#include <QProcess>

//This class carries out the task of running makeblastdb on
//the graph's nodes.
//It is a separate class because when run from the GUI, this
//process takes place in a separate thread.

class BuildBlastDatabaseWorker : public QObject
{
    Q_OBJECT

public:
    BuildBlastDatabaseWorker(QString makeblastdbCommand);
    QString m_error;

private:
    QString m_makeblastdbCommand;

public slots:
    void buildBlastDatabase();

signals:
    void finishedBuild(QString error);
};

#endif // BUILDBLASTDATABASEWORKER_H
