//Copyright 2016 Ryan Wick

//This file is part of Bandage

//Bandage is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//Bandage is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with Bandage.  If not, see <http://www.gnu.org/licenses/>.


#include "memory.h"

#include <QDir>

Memory::Memory()
{
    rememberedPath = QDir::homePath();

    commandLineCommand = NO_COMMAND;

    pathDialogIsVisible = false;
    queryPathDialogIsVisible = false;

    userSpecifiedPath = Path();
    userSpecifiedPathString = "";
    userSpecifiedPathCircular = false;

    distancePathSearchQuery1 = "";
    distancePathSearchQuery2 = "";
    distancePathSearchQuery1Path = "";
    distancePathSearchQuery2Path = "";

    terminalWidth = 80;
}



//This function clears all memory that is particular to a graph.  It should be
//called whenever a new graph is loaded.
void Memory::clearGraphSpecificMemory()
{
    userSpecifiedPath = Path();
    userSpecifiedPathString = "";
    userSpecifiedPathCircular = false;

    distanceSearchResults.clear();
    distancePathSearchQuery1 = "";
    distancePathSearchQuery2 = "";
    distancePathSearchQuery1Path = "";
    distancePathSearchQuery2Path = "";
}

